package router

import (
	"git.cestong.com.cn/cecf/cecf-golib/pkg/apollo"
	"git.cestong.com.cn/cecf/cecf-golib/pkg/gin/middleware"
	"git.cestong.com.cn/cecf/cecf-golib/pkg/log"
	"git.cestong.com.cn/cecf/cecf-golib/pkg/thirdparty"
	"git.cestong.com.cn/cecf/cecf-golib/pkg/utils"
	"git.cestong.com.cn/cecf/cecf-golib/pkg/validate"
	"git.cestong.com.cn/cecf/config-center-server/pkg/model"
	"github.com/go-playground/locales/zh"
	ut "github.com/go-playground/universal-translator"
	"github.com/go-playground/validator/v10"
	zhTranslations "github.com/go-playground/validator/v10/translations/zh"
	"gorm.io/gorm"
	"os"

	common "git.cestong.com.cn/cecf/config-center-server/common/middleware"

	"github.com/gin-gonic/gin"
	goAdminLog "github.com/go-admin-team/go-admin-core/logger"
	"github.com/go-admin-team/go-admin-core/sdk"
)

// InitRouter 路由初始化，不要怀疑，这里用到了
func InitRouter() {
	var r *gin.Engine
	h := sdk.Runtime.GetEngine()
	if h == nil {
		goAdminLog.Fatal("not found engine...")
		os.Exit(-1)
	}
	switch h.(type) {
	case *gin.Engine:
		r = h.(*gin.Engine)
	default:
		goAdminLog.Fatal("not support other engine")
		os.Exit(-1)
	}
	// 配置解析
	hxConfig, errParseConfig := parseHXConfigFromFlagFile()
	if errParseConfig != nil {
		goAdminLog.Fatalf("解析配置文件错误:%s", errParseConfig.Error())
		os.Exit(1)
	}
	// 日志配置
	if errLogSetup := setupLogger(hxConfig); errLogSetup != nil {
		goAdminLog.Fatal("设置日志:%+v, err:%s", hxConfig, errLogSetup.Error())
		return
	}

	//noRouteHandler, errCreateNoRouteHandler := buildNoRouteHandleFunc(hxConfig.HXV0.URL)
	//if errCreateNoRouteHandler != nil {
	//	goAdminLog.Fatal(errCreateNoRouteHandler.Error())
	//	os.Exit(-1)
	//}
	//r.NoRoute(noRouteHandler)

	// the jwt middleware
	authMiddleware, err := common.AuthInit()
	if err != nil {
		goAdminLog.Fatalf("JWT Init Error, %s", err.Error())
	}

	// 注册系统路由
	InitSysRouter(r, authMiddleware)
	//WARN: 这个虽然叫Example, 但是不能去掉，含有有用的go-admin路由
	InitExamplesRouter(r, authMiddleware)

	// validator
	zhTranslator := zh.New()
	universalTranslator := ut.New(zhTranslator, zhTranslator)
	reqValidator := validator.New()
	translator, findTrans := universalTranslator.GetTranslator("zh")
	if !findTrans {
		goAdminLog.Fatal("translator not found")
	}
	if errRegTrans := zhTranslations.RegisterDefaultTranslations(reqValidator, translator); errRegTrans != nil {
		goAdminLog.Fatal(errRegTrans)
	}

	db := sdk.Runtime.GetDb()["*"]
	if errMigrate := migrateModel(db); errMigrate != nil {
		goAdminLog.Fatal("合并数据库模型", errMigrate)
		os.Exit(1)
	}

	dingService := thirdparty.NewSimpleDingService(hxConfig.Ding.Keyword, hxConfig.Ding.DebugErrorAlertURL)
	dingCallback := middleware.DingMsgErrMiddlewareCallback(dingService)

	runMode := hxConfig.Common.RunMode()

	// 挂载中间件
	// panic处理
	panicMiddleware := middleware.BuildPanicMiddleware(middleware.BuildDingMsgPanicHandler(dingService))
	r.Use(panicMiddleware)
	//prometheus监控
	prometheusMetricPath := "/_metrics"
	prometheusMiddleware := middleware.NewPrometheusExporter(hxConfig.Common.ServiceID, hxConfig.HostName(),
		hxConfig.HostIP(), middleware.ExcludeByPath(prometheusMetricPath))
	r.Use(prometheusMiddleware.HandleFunc)
	r.GET(prometheusMetricPath, prometheusMiddleware.ExportMetricsHandler())
	// status告警
	statusMonitorMid := middleware.BuildStatusMonitorMiddleware(middleware.RangeStatusFilter(500, 600),
		middleware.SendDingMsg(dingService, hxConfig.Common.ServiceID, utils.HostName("defaultHost")))
	r.Use(statusMonitorMid)
	// adminID设置到context
	adminIDFinders := []middleware.AdminIDFinder{goAdminAdminIDFinder}
	if runMode == utils.DevMode {
		adminIDFinders = append(adminIDFinders, middleware.DebugAdminIDFindFunc, middleware.BuildConstAdminIDFinderFunc(1))
		debugReadBodyMiddleware := middleware.RequestBodyToContext()
		debugLogMiddleware := middleware.DebugLog()
		r.Use(debugReadBodyMiddleware).
			Use(debugLogMiddleware)
	} else {
		r.Use(middleware.BuildAccessLogMiddleware("[ACCESS]"))
	}
	adminIDSetMid := middleware.BuildSetAdminIDMiddleware(adminIDFinders...)

	// context的错误解析并转换为response.body
	errMiddleware := middleware.BuildErrMiddleware(translator, dingCallback)
	r.Use(errMiddleware)

	// routers
	//adminGroup := r.Group("/admin")
	//adminRouterMountFunc := BuildV0RouterMountFunc(db, reqValidator)
	////adminGroup.Use(authMiddleware.MiddlewareFunc()).
	////	Use(adminIDSetMid)
	//adminRouterMountFunc(adminGroup, authMiddleware)
	apolloCli := apollo.NewHTTPClient(hxConfig.Apollo)
	//if errApolloLogin := apolloCli.Login(); errApolloLogin != nil {
	//	log.Errorf("apollo login failed:%s", errApolloLogin.Error())
	//	os.Exit(1)
	//	return
	//}
	ceeRegexp := validate.NewRegExpExecuteEngine()
	ceePython := validate.NewPythonExecuteEngine(hxConfig.ConstraintExecuteEngineConfig.PythonScriptPrefixPath,
		hxConfig.ConstraintExecuteEngineConfig.Python3Path, hxConfig.ConstraintExecuteEngineConfig.TempDir)
	constraintEEMap := map[string]validate.ExecuteEngine{
		validate.EETypePython: ceePython,
		validate.EETypeRegExp: ceeRegexp,
	}
	ccApiGroup := r.Group("/api/v3")
	ccRouterMountFunc := BuildRegisterCCApi(db, reqValidator, apolloCli, constraintEEMap)
	//都需要 admin登录
	ccApiGroup.Use(authMiddleware.MiddlewareFunc()).
		Use(adminIDSetMid)
	ccRouterMountFunc(ccApiGroup, authMiddleware)
}

func migrateModel(db *gorm.DB) error {
	return db.AutoMigrate(&model.AppModel{}, &model.ConfigFileModel{}, &model.UserConfigModel{},
		&model.ConfigItemModel{}, &model.ValueConstraintModel{})
}

func setupLogger(config hxAdminConfig) error {

	logger, err := log.NewLogger(log.WithLogRunMode(config.Common.RunMode()),
		log.WithLogFile(config.Log.FileConfig), log.WithLogLevel(config.Log.Level))
	if err != nil {
		return err
	}
	log.SetLogger(logger)
	return nil
}

const (
	identityKey          = "identity"
	identityIdentityKey  = "IdentityKey"
	identityUserNameKey  = "UserName"
	identityRoleKey      = "RoleKey"
	identityUserIDKey    = "UserId"
	identityRoleIDsKey   = "RoleIds"
	identityDataScopeKey = "DataScope"
)

func goAdminAdminIDFinder(c *gin.Context) (uint, bool) {
	identity, exists := c.Get(identityKey)
	if !exists {
		return 0, false
	}
	identityMap := identity.(map[string]interface{})
	adminIDObj, adminIDFound := identityMap[identityUserIDKey]
	if !adminIDFound {
		return 0, false
	}
	adminID := uint(adminIDObj.(float64)) // jwt claims解析出来都是float
	return adminID, true
}
